@@ -3,7 +3,7 @@
 # ******************************************************************************
 #
 # Copyright (C) 2006-2011 Olivier Tilloy <olivier@tilloy.net>
-# Copyright (C) 2015-2020 Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
+# Copyright (C) 2015-2021 Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
 #
 # This file is part of the py3exiv2 distribution.
 #
@@ -85,12 +85,9 @@ class XmpTag(object):
     # strptime is not flexible enough to handle all valid Date formats, we use a
     # custom regular expression
     _time_zone_re = r'$|Z|((?P<sign>\+|-)(?P<ohours>\d{2}):(?P<ominutes>\d{2}))'
-
     _time_re = r'(?P<hours>\d{2})(:(?P<minutes>\d{2})(:(?P<seconds>\d{2})(.(?P<decimal>\d+))?)?(?P<tzd>%s))?' % _time_zone_re
-
     _date_re = re.compile(r'(?P<year>\d{4})(-(?P<month>\d{2})(-(?P<day>\d{2})(T(?P<time>%s))?)?)?' % _time_re)
 
-
     def __init__(self, key, value=None, _tag=None):
         """The tag can be initialized with an optional value which expected
         type depends on the XMP type of the tag.
@@ -98,6 +95,7 @@ class XmpTag(object):
         Args:
         key -- the key of the tag
         value -- the value of the tag
+        _tag -- an existing tag
         """
         super(XmpTag, self).__init__()
         if _tag is not None:
@@ -181,6 +179,9 @@ class XmpTag(object):
         type_ = self._tag._getExiv2Type()
         
         if type_ == 'XmpText':
+            if not isinstance(value, str):
+                raise ValueError("Type XmpText need string value, not %s" % type(value))
+
             self._tag._setTextValue(value)
 
         elif type_ in ('XmpAlt', 'XmpBag', 'XmpSeq'):
@@ -240,9 +241,23 @@ class XmpTag(object):
         type_ = self._tag._getExiv2Type()
         if type_ == 'XmpText':
             stype = self.type
-            if stype.lower().startswith('closed choice of'):
-                stype = stype[17:]
-            self.raw_value = self._convert_to_string(value, stype)
+            if stype == "Boolean":
+                self.raw_value = self._convert_to_string(value, "Boolean")
+
+            elif stype == "Date":
+                self.raw_value = self._convert_to_string(value, stype)
+
+            elif stype.lower().startswith('closed choice of'):
+                self.raw_value = self._convert_to_string(value, stype[17:])
+
+            else:
+                stype = "Text"
+                if isinstance(value, (list, tuple)):
+                    self.raw_value = [self._convert_to_string(v, stype) 
+                                        for v in value]
+
+                else:
+                    self.raw_value = self._convert_to_string(value, stype)
 
         elif type_ in ('XmpAlt', 'XmpBag', 'XmpSeq'):
             if not isinstance(value, (list, tuple)):
@@ -251,6 +266,16 @@ class XmpTag(object):
             stype = self.type[4:]
             if stype.lower().startswith('closed choice of'):
                 stype = stype[17:]
+
+            elif stype.endswith("Detail"):
+                stype = "Text"
+
+            #elif stype == "Region" or self.type == "RegionInfo":
+                #stype = "Text"
+
+            else:
+                stype = "Text"
+
             self.raw_value = [self._convert_to_string(v, stype) for v in value]
 
         elif type_ == 'LangAlt':
@@ -290,6 +315,9 @@ class XmpTag(object):
 
         Raise XmpValueError: if the conversion fails
         """
+        if value is None:
+            raise XmpValueError(value, type_)
+
         if type_ == 'Boolean':
             if value == 'True':
                 return True
@@ -305,6 +333,9 @@ class XmpTag(object):
             raise NotImplementedError('XMP conversion for type [%s]' % type_)
 
         elif type_ == 'Date':
+            if isinstance(value, datetime.date):
+                return value
+
             try:
                 v = value.replace("Z", "")
                 # New in Python-3.7
@@ -318,7 +349,6 @@ class XmpTag(object):
                 raise XmpValueError(value, type_)
 
             gd = match.groupdict()
-            print(gd)
             if gd['month'] is not None:
                 month = int(gd['month'])
 
@@ -340,7 +370,6 @@ class XmpTag(object):
             else:
                 if gd['minutes'] is None:
                     # Malformed time
-                    print(value, type_)
                     raise XmpValueError(value, type_)
 
                 if gd['seconds'] is not None:
@@ -369,9 +398,9 @@ class XmpTag(object):
                 except ValueError:
                     raise XmpValueError(value, type_)
 
-        elif type_ == 'Dimensions':
-            # TODO
-            raise NotImplementedError('XMP conversion for type [%s]' % type_)
+        elif type_ in ('RegionInfo', 'ContactInfo', 'Area', 'Dimensions'):
+            # 'Area' & 'Dimensions' are used with "Xmp.mwg-rs.Regions/mwg-rs:"
+            return value
 
         elif type_ == 'Font':
             # TODO
@@ -450,6 +479,9 @@ class XmpTag(object):
 
         Raise XmpValueError: if the conversion fails
         """
+        if value is None:
+            raise XmpValueError(value, type_)
+
         if type_ == 'Boolean':
             if isinstance(value, bool):
                 return str(value)
@@ -471,10 +503,24 @@ class XmpTag(object):
                 raise XmpValueError(value, type_)
 
         elif type_ == 'Integer':
-            if isinstance(value, int):
-                return str(value)
+            if not isinstance(value, int):
+                raise XmpValueError(value, type_)
 
-            else:
+            try:
+                return str(value)
+            except ValueError:
+                raise XmpValueError(value, type_)
+
+        elif type_ == 'Text':
+            if isinstance(value, (list, tuple)):
+                try:
+                    return " ".join([str(v) for v in value])
+                except ValueError:
+                    raise XmpValueError(value, type_)
+
+            try:
+                return str(value)
+            except ValueError:
                 raise XmpValueError(value, type_)
 
         elif type_ == 'MIMEType':
